# FB_Mfa_Spatial_D2.R 
# EU project Fisboat 502572
# Pierre Petitgas & Jean-Charles Poulard, Ifremer 
# 10 July 2007
# 

##### 
# load library
#####

library(ade4)

####
#### COMMANDS
####

#### READ DATA : Table of spatial indicators for North Sea cod - Fisboat project format
## input reference period Ref

fic<-"codNS_tab1.txt"
Tab1<-read.table(fic,header=T,sep='\t',as.is=T,colClasses=c(rep('character',4),rep('double',11)))
cat("first line of file ","\n"); Tab1[1,]

Ref<-1985:1994        
sel.refy<-is.element(unique(Tab1$Year),Ref)

cat("\n","all years ","\n"); unique(Tab1$Year)
cat("\n","reference period ","\n"); unique(Tab1$Year)[sel.refy]

#### PREPARE DATA TABLE FOR MFA

# NB columns in analysis
Nc<-length(Tab1[1,]);Nv<-(Nc-7+1)

# delete lines of Tab1 with NA values
Tab1<-Tab1[!apply(is.na(Tab1[,7:Nc]),1,sum)==Nv,] 
sel.refy<-is.element(unique(Tab1$Year),Ref)

# Ages & Years
Age<-unique(Tab1$Age); Nage<-length(Age)
Yea<-unique(Tab1$Year); Nyea<-length(Yea)
namvar<-names(Tab1[,7:14])

# modify format of tab1 for mfa: working file = matyea
 nt<-sum(Tab1$Age==Age[1])
 Tab1<-Tab1[,-  match(c("SurveyType","Abundance"),names(Tab1)) ]       # delete columns survey.type,abundance,nb.patches
 # test if all ages have same nb of years
 if (Nage!=length(Tab1[,1])/sum(Tab1$Age==Age[1])) {
   cat('different ages have different number of years',"\n")
   for (k in 1:Nage) {cat(Age[k]," ",Tab1$Year[Tab1$Age==Age[k]],sep=" ","\n") }}
 matyea<-Tab1[Tab1$Year==Yea[1],- match("Year",names(Tab1))]
 for (k in 2:Nyea) {matyea<-cbind(matyea,Tab1[Tab1$Year==Yea[k],5:12])}
 vecname<-names(Tab1)[1:3]
 for (k in 1:Nyea) {vecname<-c(vecname,paste(names(Tab1)[5:12],Yea[k],sep="."))}
 dimnames(matyea)[[2]]<-vecname
 sel.age<-!apply(is.na(matyea),1,sum)
 if (sum(sel.age)<Nage) {cat("age(s) ",Age[!sel.age]," removed because of NA values in table","\n")}
 matyea<-matyea[sel.age,]
 Age<-Age[sel.age]; Nage<-length(Age)
 # useful columns  
 dat<-matyea[,4:length(dimnames(matyea)[[2]])]
 # nb columns and tables: Nv variables, Nyea years
 nbg<-rep(Nv,length(Yea)) 
 # centre normed columns 
 dat.sc<-as.data.frame(scale(dat)); dat.sc[is.na(dat.sc)]<-0
 # create table in format used by mfa function 
 dat.ktab<-ktab.data.frame(dat.sc,nbg,rownames=Age,tabnames=Yea) 

 # array weights
 w<-rep(0,Nyea);w[sel.refy]<-1
 dat.ktab$tabw<-w 

#### PERFORM MFA

 dat.mfa<-mfa(dat.ktab,option=c("lambda1"),scannf=F,nf=Nv)
 nn<-length(dat.mfa$eig)
 cat("cumulative explained variance by the mfa components:","\n",
     cumsum(dat.mfa$eig)/sum(dat.mfa$eig),"\n")
 # plot compromise & partials individuals
 x11(); s.class(dat.mfa$lisup,dat.mfa$TL[,2],lab=row.names(dat.mfa$tab))

 # interpretation of components: correlation of each index to each component
 # count the number of times that each index has correlation > 0.5 , < -0.5
 tmp<-strsplit((rownames(dat.mfa$co)),"\\.")
 indic<-unlist(tmp)[seq(1,length(unlist(tmp)),2)]
 rankindic<-names(Tab1)[5:12]
 tabcor.p<-matrix(0,ncol=nn,nrow=length(rankindic))
 dimnames(tabcor.p)<-list(rankindic,paste("comp",1:nn,sep="."))
 tabcor.m<-matrix(0,ncol=nn,nrow=length(rankindic))
 dimnames(tabcor.m)<-list(rankindic,paste("comp",1:nn,sep="."))
 for (u in 1:nn) {
  countcor.p<-table(indic[dat.mfa$co[,u]>  0.5])
  countcor.m<-table(indic[dat.mfa$co[,u]< -0.5])
  j.p<-match(labels(countcor.p)[[1]],rankindic)
  j.m<-match(labels(countcor.m)[[1]],rankindic)
  for (k in 1:length(j.p)) { tabcor.p[j.p[k],u]<-countcor.p[k] }
  for (k in 1:length(j.m)) { tabcor.m[j.m[k],u]<-countcor.m[k] }
 }
cat("NB times corr(index,comp)>  0.5","\n"); tabcor.p
cat("\n","NB times corr(index,comp)< -0.5","\n"); tabcor.m

#### DISTANCE d2spa TO AGE SPECIFIC GRAVITY CENTRES

Ncomp<-2
# cg & v = mean and var of $lisup for each age on each axis 
#                          in the ref period
 cgr<-matrix(NA,nrow=Nage,ncol=Ncomp); row.names(cgr)<-Age
 vr<-matrix(NA,nrow=Nage,ncol=Ncomp); row.names(vr)<-Age
 for (k in 1:Nage) {
  ll<-seq(1:length(dat.mfa$lisup[,1]))[dat.mfa$TL$L==k]  
  cgr[k,]<-apply(dat.mfa$lisup[ll[sel.refy],1:Ncomp],2,mean)
  vr[k,]<-apply(dat.mfa$lisup[ll[sel.refy],1:Ncomp],2,var)
 }
# d2 for each age: ($lisup-cg)^2/v
 d2a<-matrix(NA,nrow=Nyea,ncol=Nage);dimnames(d2a)<-list(paste(Yea),Age)
 xx<-NA
 for (i in 1:Nyea) {
  ll<-seq(1:length(dat.mfa$lisup[,1]))[dat.mfa$TL$T==i]
  d2a[i,]<-apply( (dat.mfa$lisup[ll,1:Ncomp]-cgr)^2/vr,1,sum ) 
  xx<-c(xx,(dat.mfa$lisup[ll,1:Ncomp]-cgr)^2/vr)
 }
# d2 for all ages: sum over ages
 d2<-cbind(Yea,apply(d2a,1,sum)); dimnames(d2)<-list(NULL,c('year','d2'))

x11(); plot(d2[,1],d2[,2],type="b",pch=18,xlab='',
            ylab=paste('Multivariate spatial index'),cex.lab=1.1 )

### SAVE

mfa.res<-list("eig"=dat.mfa$eig,"cor.plus"=tabcor.p,"cor.minus"=tabcor.m,
           "refyears"=Ref,"ncomp"=Ncomp,"cgr"=cgr,"vr"=vr,"d2a"=d2a,"d2"=d2)

mfa.res<-c(cs=substring(fic[kfic],1,5),mfa.res)
assign(paste("mfa.",substring(fic[kfic],1,5),sep=""),mfa.res)

# write dmul index to file on disk
write.table(file=paste(substring(fic,1,5),"_mfa_index",".txt",sep=""),
            mfa.res$d2,sep="\t",row.names=F)


### free memory

rm(pat,fic,kfic,Refy,Tab1,Ref,sel.refy,Ncomp,mfa.res,corp,corm,interpret.comp)


